"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Rx = require("rxjs/Rx");
const XRegExp = require("xregexp");
const gensequence_1 = require("gensequence");
const Text = require("./text");
const fileReader_1 = require("./fileReader");
const cspell_lib_1 = require("cspell-lib");
const path = require("path");
const fs_extra_1 = require("fs-extra");
const rxjs_from_iterable_1 = require("rxjs-from-iterable");
const Trie = require("cspell-trie");
const HR = require("hunspell-reader");
const regNonWordOrSpace = XRegExp("[^\\p{L}' ]+", 'gi');
const regExpSpaceOrDash = /(?:\s+)|(?:-+)/g;
const regExpRepeatChars = /(.)\1{3,}/i;
function normalizeWords(lines) {
    return lines.flatMap(line => lineToWords(line).toArray());
}
exports.normalizeWords = normalizeWords;
function lineToWords(line) {
    // Remove punctuation and non-letters.
    const filteredLine = line.replace(regNonWordOrSpace, '|');
    const wordGroups = filteredLine.split('|');
    const words = gensequence_1.genSequence(wordGroups)
        .concatMap(a => [a, ...a.split(regExpSpaceOrDash)])
        .concatMap(a => splitCamelCase(a))
        .map(a => a.trim())
        .filter(a => !!a)
        .filter(s => !regExpRepeatChars.test(s))
        .map(a => a.toLowerCase())
        .reduceToSequence((s, w) => s.add(w), new Set());
    return words;
}
exports.lineToWords = lineToWords;
function splitCamelCase(word) {
    const splitWords = Text.splitCamelCaseWord(word);
    // We only want to preserve this: "New York" and not "Namespace DNSLookup"
    if (splitWords.length > 1 && regExpSpaceOrDash.test(word)) {
        return gensequence_1.genSequence(splitWords).concatMap(w => w.split(regExpSpaceOrDash));
    }
    return splitWords;
}
function compileSetOfWords(lines) {
    const set = normalizeWords(lines)
        .reduce((s, w) => s.add(w), new Set())
        .toPromise();
    return Promise.all([set]).then(a => a[0]);
}
exports.compileSetOfWords = compileSetOfWords;
function compileWordList(filename, destFilename) {
    const getWords = () => regHunspellFile.test(filename) ? readHunspellFiles(filename) : fileReader_1.lineReaderRx(filename);
    const destDir = path.dirname(destFilename);
    return fs_extra_1.mkdirp(destDir).then(() => compileSetOfWords(getWords()))
        .then(set => {
        const data = gensequence_1.genSequence(set)
            .map(a => a + '\n')
            .toArray()
            .sort()
            .join('');
        return cspell_lib_1.writeToFile(destFilename, data);
    });
}
exports.compileWordList = compileWordList;
function normalizeWordsToTrie(words) {
    const result = normalizeWords(words)
        .reduce((node, word) => Trie.insert(word, node), {})
        .toPromise();
    return result;
}
exports.normalizeWordsToTrie = normalizeWordsToTrie;
function compileWordListToTrieFile(words, destFilename) {
    const destDir = path.dirname(destFilename);
    const dir = fs_extra_1.mkdirp(destDir);
    const root = normalizeWordsToTrie(words);
    const data = Rx.Observable.zip(dir, root, (_, b) => b)
        .map(node => Trie.serializeTrie(node, { base: 32, comment: 'Built by cspell-tools.' }))
        .flatMap(seq => rxjs_from_iterable_1.observableFromIterable(seq));
    return cspell_lib_1.writeToFileRxP(destFilename, data.bufferCount(1024).map(a => a.join('')));
}
exports.compileWordListToTrieFile = compileWordListToTrieFile;
const regHunspellFile = /\.(dic|aff)$/i;
function readHunspellFiles(filename) {
    const dicFile = filename.replace(regHunspellFile, '.dic');
    const affFile = filename.replace(regHunspellFile, '.aff');
    const reader = HR.HunspellReader.createFromFiles(affFile, dicFile);
    const r = Rx.Observable.fromPromise(reader)
        .flatMap(reader => reader.readWordsRx())
        .map(aff => aff.word);
    return r;
}
function compileTrie(filename, destFilename) {
    const words = regHunspellFile.test(filename) ? readHunspellFiles(filename) : fileReader_1.lineReaderRx(filename);
    return compileWordListToTrieFile(words, destFilename);
}
exports.compileTrie = compileTrie;
//# sourceMappingURL=wordListCompiler.js.map